@@ -6,6 +6,7 @@ import yaml
 import traceback
 import threading
 from typing import List, Dict, Any
+from os.path import exists
 
 # External libraries
 import pandas as pd
@@ -25,6 +26,7 @@ class API:
     :exchange: The market the API trades on. Ignored if the API is not a broker.
     """
 
+    # List of supported intervals
     interval_list = [
         Interval.MIN_1,
         Interval.MIN_5,
@@ -33,8 +35,10 @@ class API:
         Interval.HR_1,
         Interval.DAY_1,
     ]
-
+    # Name of the exchange this API trades on
     exchange = ""
+    # List of attributes that are required to be in the secret file
+    req_keys = []
 
     def __init__(self, path: str = None):
         """
@@ -53,26 +57,34 @@ class API:
         :path: path to the YAML file containing credentials to communicate with the API.
             If not specified, defaults to './secret.yaml'
         """
+        config = {}
 
         if path is None:
             path = "./secret.yaml"
-        # Check if file exists
-        yml_file = Path(path)
-        if not yml_file.is_file() and not self.create_secret(path):
-            debugger.debug("Broker not initalized with account information.")
+        # Check if file exists. If not, create a secret file
+        if not exists(path):
+            config = self.create_secret(path)
         else:
+            # Open file
             with open(path, "r") as stream:
-                self.config = yaml.safe_load(stream)
+                config = yaml.safe_load(stream)
+                # Check if the file contains all the required parameters
+                if any(key not in config for key in self.req_keys):
+                    config.update(self.create_secret(path))
 
-        self.timestamp = now()
+        with open(path, "w") as f:
+            yaml.dump(config, f)
+
+        self.config = config
 
     def create_secret(self, path: str):
         """
         This method is called when the yaml file with credentials
-        is not found."""
+        is not found. It returns a dictionary containing the necessary credentials.
+        """
         # raise Exception(f"{path} was not found.")
-        debugger.warning(f"Assuming API does not need account information.")
-        return False
+        debugger.warning("Assuming API does not need account information.")
+        return None
 
     def refresh_cred(self):
         """
@@ -81,7 +93,7 @@ class API:
         """
         debugger.info(f"Refreshing credentials for {type(self).__name__}.")
 
-    def setup(self, stats: Stats, trader_main=None) -> None:
+    def setup(self, stats: Stats, account: Account, trader_main=None) -> None:
         """
         This function is called right before the algorithm begins,
         and initializes several runtime parameters like
@@ -93,6 +105,7 @@ class API:
         self.trader_main = trader_main
         self.stats = stats
         self.stats.timestamp = now()
+        self.account = account
 
         min_interval = None
         for sym in stats.watchlist_cfg:
@@ -114,9 +127,8 @@ class API:
             ):
                 min_interval = stats.watchlist_cfg[sym]["interval"]
 
-        self.interval = stats.watchlist_cfg
         self.poll_interval = min_interval
-        debugger.debug(f"Interval: {self.interval}")
+
         debugger.debug(f"Poll Interval: {self.poll_interval}")
         debugger.debug(f"{type(self).__name__} setup finished")
 
@@ -182,8 +194,9 @@ class API:
         # intervals that needs to be called now, fetch the latest data
 
         df_dict = {}
-        for sym in self.interval:
-            inter = self.interval[sym]["interval"]
+        for sym in self.stats.watchlist_cfg:
+            inter = self.stats.watchlist_cfg[sym]["interval"]
+
             if is_freq(self.stats.timestamp, inter):
                 n = self.stats.timestamp
                 latest = self.fetch_price_history(
@@ -289,7 +302,7 @@ class API:
 
         :param symbol: Stock symbol. Cannot use crypto.
         :returns: A dict with the following keys and values:
-            - id: ID of the option chain
+            - chain_id: ID of the option chain
             - exp_dates: List of expiration dates as datetime objects
             - multiplier: Multiplier of the option, usually 100
         """
@@ -297,11 +310,12 @@ class API:
             f"{type(self).__name__} does not support this streamer method: `fetch_chain_info`."
         )
 
-    def fetch_chain_data(self, symbol: str):
+    def fetch_chain_data(self, symbol: str, date):
         """
         Returns the option chain for the specified symbol.
 
         :param symbol: Stock symbol. Cannot use crypto.
+        :param date: Expiration date.
         :returns: A dataframe in the following format:
 
                     exp_date strike  type
@@ -333,12 +347,14 @@ class API:
         Hours are based on the exchange specified in the class's 'exchange' attribute.
 
         :returns: A dictionary with the following keys and values:
-            - open: Time the market opens in UTC timezone.
-            - close: Time the market closes in UTC timezone.
+            - is_open: Boolean indicating whether the market is open or closed
+            - open_at: Time the market opens in UTC timezone.
+            - close_at: Time the market closes in UTC timezone.
         """
-        raise NotImplementedError(
-            f"{type(self).__name__} does not support this broker method: `fetch_market_hours`."
-        )
+        # raise NotImplementedError(
+        #     f"{type(self).__name__} does not support this broker method: `fetch_market_hours`."
+        # )
+        return {"is_open": True, "open_at": None, "close_at": None}
 
     # ------------- Broker methods ------------- #
 
@@ -424,8 +440,7 @@ class API:
 
         :returns: A dictionary with the following keys and values:
             - type: 'STOCK'
-            - id: ID of the order
-            - symbol: Ticker of stock
+            - order_id: ID of the order
             - quantity: Quantity ordered
             - filled_quantity: Quantity filled so far
             - side: 'buy' or 'sell'
@@ -446,8 +461,7 @@ class API:
 
         :returns: A dictionary with the following keys and values:
             - type: 'OPTION'
-            - id: ID of the order
-            - symbol: OCC symbol of option
+            - order_id: ID of the order
             - quantity: Quantity ordered
             - filled_quantity: Quantity filled so far
             - side: 'buy' or 'sell'
@@ -468,8 +482,7 @@ class API:
 
         :returns: A dictionary with the following keys and values:
             - type: 'CRYPTO'
-            - id: ID of the order
-            - symbol: Ticker of crypto
+            - order_id: ID of the order
             - quantity: Quantity ordered
             - filled_quantity: Quantity filled so far
             - side: 'buy' or 'sell'
@@ -488,32 +501,28 @@ class API:
 
         returns: A list of dictionaries with the following keys and values:
             For stocks and crypto:
-                - type: "STOCK" or "CRYPTO"
+                - order_type: "STOCK" or "CRYPTO"
                 - symbol: Symbol of asset
                 - quantity: Quantity ordered
                 - filled_qty: Quantity filled
-                - id: ID of order
+                - order_id: ID of order
                 - time_in_force: Time in force
                 - status: Status of the order
                 - side: 'buy' or 'sell'
                 - filled_time: Time the order was filled
                 - filled_price: Price the order was filled at
             For options:
-                - type: "OPTION",
+                - order_type: "OPTION",
                 - symbol: OCC symbol of option
                 - base_symbol:
                 - quantity: Quantity ordered
                 - filled_qty: Quantity filled
                 - filled_time: Time the order was filled
                 - filled_price: Price the order was filled at
-                - id: ID of order
+                - order_id: ID of order
                 - time_in_force: Time in force
                 - status: Status of the order
-                - legs: A list of dictionaries with keys:
-                    - id: id of leg
-                    - side: 'buy' or 'sell'
-                    Harvest does not support buying multiple options in a single transaction,
-                    so legs will always have a length of 1.
+                - side: 'buy' or 'sell'
 
         """
         debugger.error(
@@ -543,7 +552,7 @@ class API:
         :extended:  'False' by default
 
         :returns: A dictionary with the following keys and values:
-            - id: ID of order
+            - order_id: ID of order
             - symbol: symbol of asset
             Raises an exception if order fails.
         """
@@ -571,7 +580,7 @@ class API:
         :extended:  'False' by default
 
         :returns: A dictionary with the following keys and values:
-            - id: ID of order
+            - order_id: ID of order
             - symbol: symbol of asset
             Raises an exception if order fails.
         """
@@ -603,8 +612,7 @@ class API:
         :option_type:      'call' or 'put'
 
         :returns: A dictionary with the following keys and values:
-            - type: 'OPTION'
-            - id: ID of order
+            - order_id: ID of order
             - symbol: symbol of asset
             Raises an exception if order fails.
         """
@@ -612,6 +620,21 @@ class API:
             f"{type(self).__name__} does not support this broker method: `order_option_limit`."
         )
 
+    def cancel_stock_order(self, order_id):
+        raise NotImplementedError(
+            f"{type(self).__name__} does not support this broker method: `cancel_stock_order`."
+        )
+
+    def cancel_crypto_order(self, order_id):
+        raise NotImplementedError(
+            f"{type(self).__name__} does not support this broker method: `cancel_crypto_order`."
+        )
+
+    def cancel_option_order(self, order_id):
+        raise NotImplementedError(
+            f"{type(self).__name__} does not support this broker method: `cancel_option_order`."
+        )
+
     # -------------- Built-in methods -------------- #
     # These do not need to be re-implemented in a subclass
 
@@ -705,6 +728,17 @@ class API:
         else:
             debugger.error(f"Invalid asset type for {symbol}")
 
+    def cancel(self, order_id):
+        for o in self.account.orders.orders:
+            if o.order_id == order_id:
+                asset_type = symbol_type(o.symbol)
+                if asset_type == "STOCK":
+                    self.cancel_stock_order(order_id)
+                elif asset_type == "CRYPTO":
+                    self.cancel_crypto_order(order_id)
+                elif asset_type == "OPTION":
+                    self.cancel_option_order(order_id)
+
     # def buy_option(self, symbol: str, quantity: int = 0, in_force: str = "gtc"):
     #     """
     #     Buys the specified option.
@@ -792,39 +826,10 @@ class API:
     def data_to_occ(
         self, symbol: str, date: dt.datetime, option_type: str, price: float
     ):
-        """
-        Converts data into a OCC format string
-        """
-        occ = symbol + ((6 - len(symbol)) * " ")
-        occ = occ + date.strftime("%y%m%d")
-        occ = occ + "C" if option_type == "call" else occ + "P"
-        occ = occ + f"{int(price*1000):08}"
-        return occ
+        return data_to_occ(symbol, date, option_type, price)
 
     def occ_to_data(self, symbol: str):
-        original_symbol = symbol
-        debugger.debug(f"Converting {symbol} to data")
-        try:
-            sym = ""
-            symbol = symbol.replace(" ", "")
-            i = 0
-            while symbol[i].isalpha():
-                i += 1
-            sym = symbol[0:i]
-            symbol = symbol[i:]
-            debugger.debug(f"{sym}, {symbol}")
-
-            date = dt.datetime.strptime(symbol[0:6], "%y%m%d")
-            debugger.debug(f"{date}, {symbol}")
-            option_type = "call" if symbol[6] == "C" else "put"
-            debugger.debug(f"{option_type}, {symbol}")
-            price = float(symbol[7:]) / 1000
-            debugger.debug(f"{price}, {symbol}")
-            return sym, date, option_type, price
-        except Exception as e:
-            debugger.error(f"Error parsing OCC symbol: {original_symbol}, {e}")
-            # return None, None, None, None
-            raise Exception(f"Error parsing OCC symbol: {original_symbol}, {e}")
+        return occ_to_data(symbol)
 
     def current_timestamp(self):
         return self.timestamp
@@ -842,8 +847,8 @@ class StreamAPI(API):
         self.block_queue = {}
         self.first = True
 
-    def setup(self, interval: Dict, trader_main=None) -> None:
-        super().setup(interval, trader_main)
+    def setup(self, stats, account, trader_main=None) -> None:
+        super().setup(stats, account, trader_main)
         self.blocker = {}
 
     def start(self):
@@ -861,8 +866,8 @@ class StreamAPI(API):
         if self.first:
             self.needed = [
                 sym
-                for sym in self.interval
-                if is_freq(now(), self.interval[sym]["interval"])
+                for sym in self.stats.watchlist_cfg
+                if is_freq(now(), self.stats.watchlist_cfg[sym]["interval"])
             ]
             self.stats.timestamp = df_dict[got[0]].index[0]
 
@@ -908,7 +913,7 @@ class StreamAPI(API):
         for n in self.needed:
             data = pd.DataFrame(
                 {"open": 0, "high": 0, "low": 0, "close": 0, "volume": 0},
-                index=[self.timestamp],
+                index=[self.stats.timestamp],
             )
 
             data.columns = pd.MultiIndex.from_product([[n], data.columns])
