@@ -19,9 +19,16 @@ class Robinhood(API):
 
     interval_list = [Interval.SEC_15, Interval.MIN_5, Interval.HR_1, Interval.DAY_1]
     exchange = "NASDAQ"
+    req_keys = ["robin_username", "robin_password", "robin_mfa"]
 
     def __init__(self, path=None):
         super().__init__(path)
+
+        if self.config is None:
+            raise Exception(
+                f"Account credentials not found! Expected file path: {path}"
+            )
+
         self.login()
 
     def login(self):
@@ -42,15 +49,18 @@ class Robinhood(API):
         debugger.debug("Logged into Robinhood...")
 
     # @API._run_once
-    def setup(self, stats, trader_main=None):
+    def setup(self, stats, account, trader_main=None):
 
-        super().setup(stats, trader_main)
+        super().setup(stats, account, trader_main)
 
         # Robinhood only supports 15SEC, 1MIN interval for crypto
-        for sym in self.stats.interval:
-            if not is_crypto(sym) and self.interval[sym]["interval"] < Interval.MIN_5:
+        for sym in self.stats.watchlist_cfg:
+            if (
+                not is_crypto(sym)
+                and self.stats.watchlist_cfg[sym]["interval"] < Interval.MIN_5
+            ):
                 raise Exception(
-                    f'Interval {self.interval[sym]["interval"]} is only supported for crypto'
+                    f'Interval {self.stats.watchlist_cfg[sym]["interval"]} is only supported for crypto'
                 )
 
         # self.__watch_stock = []
@@ -81,7 +91,7 @@ class Robinhood(API):
     #         if 'error' in ret or ret == None or (type(ret) == list and len(ret) == 0):
     #             continue
     #         df_tmp = pd.DataFrame.from_dict(ret)
-    #         df_tmp = self._format_df(df_tmp, [s], self.interval).iloc[[-1]]
+    #         df_tmp = self._format_df(df_tmp, [s], self.stats.watchlist_cfg).iloc[[-1]]
     #         df[s] = df_tmp
 
     #     return df
@@ -96,7 +106,7 @@ class Robinhood(API):
     #             span='hour',
     #             )
     #         df_tmp = pd.DataFrame.from_dict(ret)
-    #         df_tmp = self._format_df(df_tmp, ['@'+s], self.interval).iloc[[-1]]
+    #         df_tmp = self._format_df(df_tmp, ['@'+s], self.stats.watchlist_cfg).iloc[[-1]]
     #         df['@'+s] = df_tmp
 
     #     return df
@@ -147,24 +157,32 @@ class Robinhood(API):
 
         delta = end - start
         delta = delta.total_seconds()
-        delta = delta / 3600
-        if interval == Interval.DAY_1 and delta < 24:
-            return df
-        if delta < 1 and interval in [Interval.SEC_15, Interval.MIN_1]:
+        delta_hours = int(delta / 3600)
+
+        # 15SEC interval must be capped to 1 hour.
+        if interval == Interval.SEC_15:
             span = "hour"
-        elif delta < 24 or interval in [
-            Interval.MIN_5,
+        # 1MIN, 5MIN interval must be capped to 1 day.
+        elif interval in [Interval.MIN_1, Interval.MIN_5]:
+            span = "day"
+            span = "day"
+        # Other intervals have varying spans
+        elif interval in [
             Interval.MIN_15,
             Interval.MIN_30,
             Interval.HR_1,
         ]:
-            span = "day"
-        elif delta < 24 * 28:
-            span = "month"
-        elif delta < 24 * 300:
-            span = "year"
-        else:
-            span = "5year"
+            if delta_hours <= 24:
+                span = "day"
+            else:
+                span = "month"
+        elif interval in [Interval.DAY_1]:
+            if delta_hours <= 24:
+                span = "day"
+            elif delta_hours <= 720:
+                span = "month"
+            else:
+                span = "year"
 
         if symbol[0] == "@":
             ret = rh.get_crypto_historicals(
@@ -183,7 +201,7 @@ class Robinhood(API):
     def fetch_chain_info(self, symbol: str):
         ret = rh.get_chains(symbol)
         return {
-            "id": "n/a",
+            "chain_id": "n/a",
             "exp_dates": [str_to_date(s) for s in ret["expiration_dates"]],
             "multiplier": ret["trade_value_multiplier"],
         }
@@ -201,8 +219,8 @@ class Robinhood(API):
         ret = rh.find_tradable_options(symbol, date_to_str(date))
         exp_date = []
         strike = []
-        type = []
-        id = []
+        option_type = []
+        option_id = []
         occ = []
         for entry in ret:
             date = entry["expiration_date"]
@@ -211,18 +229,18 @@ class Robinhood(API):
             exp_date.append(date)
             price = float(entry["strike_price"])
             strike.append(price)
-            type.append(entry["type"])
-            id.append(entry["id"])
+            option_type.append(entry["type"])
+            option_id.append(entry["id"])
 
-            occ.append(self.data_to_occ(symbol, date, type[-1], price))
+            occ.append(data_to_occ(symbol, date, option_type[-1], price))
 
         df = pd.DataFrame(
             {
                 "occ_symbol": occ,
                 "exp_date": exp_date,
                 "strike": strike,
-                "type": type,
-                "id": id,
+                "type": option_type,
+                # "id": option_id,
             }
         )
         df = df.set_index("occ_symbol")
@@ -247,6 +265,16 @@ class Robinhood(API):
             "bid": float(ret["bid_price"]),
         }
 
+    @API._exception_handler
+    def fetch_market_hours(self, date: datetime.date):
+        ret = rh.get_market_hours(date.strftime("%Y-%m-%d"))
+
+        return {
+            "is_open": ret["is_open"],
+            "open": ret["opens_at"],
+            "close": ret["closes_at"],
+        }
+
     # ------------- Broker methods ------------- #
 
     @API._exception_handler
@@ -317,17 +345,17 @@ class Robinhood(API):
             )
         return pos
 
-    @API._exception_handler
-    def update_option_positions(self, positions: List[Any]):
-        for r in positions:
-            sym, date, type, price = self.occ_to_data(r["symbol"])
-            upd = rh.get_option_market_data(
-                sym, date.strftime("%Y-%m-%d"), str(price), type
-            )
-            upd = upd[0][0]
-            r["current_price"] = float(upd["adjusted_mark_price"])
-            r["market_value"] = float(upd["adjusted_mark_price"]) * r["quantity"]
-            r["cost_basis"] = r["avg_price"] * r["quantity"]
+    # @API._exception_handler
+    # def update_option_positions(self, positions: List[Any]):
+    #     for r in positions:
+    #         sym, date, type, price = self.occ_to_data(r["symbol"])
+    #         upd = rh.get_option_market_data(
+    #             sym, date.strftime("%Y-%m-%d"), str(price), type
+    #         )
+    #         upd = upd[0][0]
+    #         r["current_price"] = float(upd["adjusted_mark_price"])
+    #         r["market_value"] = float(upd["adjusted_mark_price"]) * r["quantity"]
+    #         r["cost_basis"] = r["avg_price"] * r["quantity"]
 
     @API._exception_handler
     def fetch_account(self):
@@ -354,8 +382,7 @@ class Robinhood(API):
             filled_price = None
         return {
             "type": "STOCK",
-            "id": ret["id"],
-            "symbol": ret["symbol"],
+            "order_id": ret["id"],
             "quantity": ret["qty"],
             "filled_quantity": ret["filled_qty"],
             "side": ret["side"],
@@ -374,15 +401,14 @@ class Robinhood(API):
         if len(executions) > 0:
             filled_time = self._rh_datestr_to_datetime(executions[0]["timestamp"])
             filled_time = filled_time.replace(tzinfo=pytz.utc)
-            filled_price = float(executions[0]["effective_price"])
+            filled_price = float(executions[0]["price"])
         else:
             filled_time = None
             filled_price = None
         return {
             "type": "OPTION",
-            "id": ret["id"],
-            "symbol": ret["chain_symbol"],
-            "qty": ret["quantity"],
+            "order_id": ret["id"],
+            "quantity": ret["quantity"],
             "filled_qty": ret["processed_quantity"],
             "side": ret["legs"][0]["side"],
             "time_in_force": ret["time_in_force"],
@@ -407,8 +433,7 @@ class Robinhood(API):
 
         return {
             "type": "CRYPTO",
-            "id": ret["id"],
-            "symbol": ret["symbol"],
+            "order_id": ret["id"],
             "quantity": float(ret["quantity"]),
             "filled_qty": float(ret["cumulative_quantity"]),
             "side": ret["side"],
@@ -426,11 +451,11 @@ class Robinhood(API):
             sym = rh.get_symbol_by_url(r["instrument"])
             queue.append(
                 {
-                    "type": "STOCK",
+                    "order_type": "STOCK",
                     "symbol": sym,
                     "quantity": r["quantity"],
                     "filled_qty": r["cumulative_quantity"],
-                    "id": r["id"],
+                    "order_id": r["id"],
                     "time_in_force": r["time_in_force"],
                     "status": r["state"],
                     "side": r["side"],
@@ -442,7 +467,8 @@ class Robinhood(API):
         ret = rh.get_all_open_option_orders()
         for r in ret:
             debugger.debug(r)
-            legs = [{"id": l["id"], "side": l["side"]} for l in r["legs"]]
+            # Assume there is only 1 leg per option order
+            leg = r["legs"][0]
             date = r["legs"][0]["expiration_date"]
             date = dt.datetime.strptime(date, "%Y-%m-%d")
             s = self.data_to_occ(
@@ -453,7 +479,7 @@ class Robinhood(API):
             )
             queue.append(
                 {
-                    "type": "OPTION",
+                    "order_type": "OPTION",
                     "symbol": s,
                     "base_symbol": r["chain_symbol"],
                     "quantity": r["quantity"],
@@ -461,7 +487,8 @@ class Robinhood(API):
                     "id": r["id"],
                     "time_in_force": r["time_in_force"],
                     "status": r["state"],
-                    "legs": legs,
+                    "order_id": leg["id"],
+                    "side": leg["side"],
                     "filled_time": None,
                     "filled_price": None,
                 }
@@ -471,11 +498,11 @@ class Robinhood(API):
             sym = rh.get_symbol_by_url(r["instrument"])
             queue.append(
                 {
-                    "type": "CRYPTO",
+                    "order_type": "CRYPTO",
                     "symbol": sym,
                     "quantity": r["quantity"],
                     "filled_qty": r["cumulative_quantity"],
-                    "id": r["id"],
+                    "order_id": r["id"],
                     "time_in_force": r["time_in_force"],
                     "status": r["state"],
                     "side": r["side"],
@@ -515,8 +542,7 @@ class Robinhood(API):
                     limitPrice=limit_price,
                 )
             return {
-                "type": "STOCK",
-                "id": ret["id"],
+                "order_id": ret["id"],
                 "symbol": symbol,
             }
         except:
@@ -533,6 +559,7 @@ class Robinhood(API):
         extended: bool = False,
     ):
         ret = None
+        symbol = symbol[1:]
         try:
             if side == "buy":
                 ret = rh.order_buy_crypto_limit(
@@ -549,13 +576,14 @@ class Robinhood(API):
                     limitPrice=limit_price,
                 )
             return {
-                "type": "CRYPTO",
-                "id": ret["id"],
+                "order_id": ret["id"],
                 "symbol": "@" + symbol,
             }
         except:
-            debugger.error("Error while placing order.\nReturned: {ret}", exc_info=True)
-            raise Exception("Error while placing order.")
+            debugger.error(
+                f"Error while placing order.\nReturned: {ret}", exc_info=True
+            )
+            raise Exception(f"Error while placing order.")
 
     def order_option_limit(
         self,
@@ -595,14 +623,27 @@ class Robinhood(API):
                     timeInForce=in_force,
                 )
             return {
-                "type": "OPTION",
-                "id": ret["id"],
+                "order_id": ret["id"],
                 "symbol": symbol,
             }
         except:
-            debugger.error("Error while placing order.\nReturned: {ret}", exc_info=True)
+            debugger.error(
+                f"Error while placing order.\nReturned: {ret}", exc_info=True
+            )
             raise Exception("Error while placing order")
 
+    def cancel_stock_order(self, order_id):
+        ret = rh.cancel_stock_order(order_id)
+        debugger.debug(ret)
+
+    def cancel_option_order(self, order_id):
+        ret = rh.cancel_option_order(order_id)
+        debugger.debug(ret)
+
+    def cancel_crypto_order(self, order_id):
+        ret = rh.cancel_crypto_order(order_id)
+        debugger.debug(ret)
+
     def _format_df(
         self, df: pd.DataFrame, watch: List[str], interval: str, latest: bool = False
     ):
@@ -635,7 +676,7 @@ class Robinhood(API):
         date_str = date_str[:-3] + date_str[-2:]
         return dt.datetime.strptime(date_str, "%Y-%m-%dT%H:%M:%S.%f%z")
 
-    def create_secret(self, path):
+    def create_secret(self):
         import harvest.wizard as wizard
 
         w = wizard.Wizard()
@@ -735,17 +776,8 @@ class Robinhood(API):
 
         w.println(f"All steps are complete now 🎉. Generating secret.yml...")
 
-        d = {
+        return {
             "robin_mfa": f"{mfa}",
             "robin_username": f"{username}",
             "robin_password": f"{password}",
         }
-
-        with open(path, "w") as file:
-            yml = yaml.dump(d, file)
-
-        w.println(
-            f"secret.yml has been created! Make sure you keep this file somewhere secure and never share it with other people."
-        )
-
-        return True
